@@ -77,6 +77,7 @@ public:
             mAllowMixed(allowMixed),
             mCache(KeyCache::instance()),
             mDialogWindowFlags(Qt::WindowFlags()),
+            mPreferredProtocol(GpgME::UnknownProtocol),
             mMinimumValidity(GpgME::UserID::Marginal),
             mCompliance(Formatting::complianceMode())
     {
@@ -134,7 +135,7 @@ public:
             return;
         }
 
-        // Internally we work with normalized addresses. Normalisation
+        // Internally we work with normalized addresses. Normalization
         // matches the gnupg one.
         for (const auto &addr :addresses) {
             // PGP Uids are defined to be UTF-8 (RFC 4880 §5.11)
@@ -173,12 +174,12 @@ public:
         }
         for (CryptoMessageFormat fmt: mOverrides.keys()) {
             // Iterate over the crypto message formats
-            if (mFormat != AutoFormat && mFormat != fmt) {
+            if (mFormat != AutoFormat && mFormat != fmt && fmt != AutoFormat) {
                 // Skip overrides for the wrong format
                 continue;
             }
             for (const auto &addr: mOverrides[fmt].keys()) {
-                // For all adress overrides of this format.
+                // For all address overrides of this format.
                 for (const auto &fprOrId: mOverrides[fmt][addr]) {
                     // For all the keys configured for this address.
                     const auto key = mCache->findByKeyIDOrFingerprint(fprOrId.toUtf8().constData());
@@ -189,7 +190,7 @@ public:
                     }
 
                     // Now add it to the resolved keys and remove it from our list
-                    // of unsresolved keys.
+                    // of unresolved keys.
                     QMap<CryptoMessageFormat, QMap <QString, std::vector<GpgME::Key> > > *targetMap;
                     if (mRecipients.contains(addr)) {
                         targetMap = &mEncKeys;
@@ -201,11 +202,21 @@ public:
                         continue;
                     }
 
-                    auto recpMap = targetMap->value(fmt);
+                    CryptoMessageFormat resolvedFmt = fmt;
+                    if (fmt == AutoFormat) {
+                        // Take the format from the key.
+                        if (key.protocol() == GpgME::OpenPGP) {
+                            resolvedFmt = AnyOpenPGP;
+                        } else {
+                            resolvedFmt = AnySMIME;
+                        }
+                    }
+
+                    auto recpMap = targetMap->value(resolvedFmt);
                     auto keys = recpMap.value(addr);
                     keys.push_back(key);
                     recpMap.insert(addr, keys);
-                    targetMap->insert(fmt, recpMap);
+                    targetMap->insert(resolvedFmt, recpMap);
 
                     // Now we can remove it from our unresolved lists.
                     if (key.protocol() == GpgME::OpenPGP) {
@@ -213,6 +224,7 @@ public:
                     } else {
                         mUnresolvedCMS.removeAll(addr);
                     }
+                    qCDebug(LIBKLEO_LOG) << "Override" << addr << cryptoMessageFormatToString (fmt) << fprOrId;
                 }
             }
         }
@@ -441,10 +453,10 @@ public:
         QStringList unresolvedRecp;
 
         if (mEncrypt) {
-            // Use all unresolved recpients.
+            // Use all unresolved recipients.
             if (!cmsOnly && !pgpOnly) {
                 if (mFormat & AutoFormat) {
-                    // In Auto Format we can now remove recpients that could
+                    // In Auto Format we can now remove recipients that could
                     // be resolved either through CMS or PGP
                     for (const auto &addr: mUnresolvedPGP) {
                         if (mUnresolvedCMS.contains(addr)) {
@@ -466,9 +478,9 @@ public:
                     if (!resolvedRecp.contains(addr) || !resolvedRecp[addr].size()) {
                         resolvedRecp.insert(addr, map[addr]);
                     } else {
-                        for (const auto &k: map[addr]) {
-                            resolvedRecp[addr].push_back(k);
-                        }
+                        qCDebug(LIBKLEO_LOG) << "Replacing resolved keys for" << addr
+                                             << "with keys from new format.";
+                        resolvedRecp[addr] = map[addr];
                     }
                 }
             }
@@ -482,7 +494,7 @@ public:
         // Start with the protocol for which every keys could be found.
         GpgME::Protocol presetProtocol = pgpOnly ? GpgME::OpenPGP :
                                          cmsOnly ? GpgME::CMS :
-                                         GpgME::UnknownProtocol;
+                                         mPreferredProtocol;
 
         mDialog = std::shared_ptr<NewKeyApprovalDialog>(new NewKeyApprovalDialog(resolvedSig,
                                                                                  resolvedRecp,
@@ -571,10 +583,11 @@ public:
     bool mEncrypt, mSign, mNag;
     bool mAllowMixed;
     // The cache is needed as a member variable to avoid rebuilding
-    // it beteween calls if we are the only user.
+    // it between calls if we are the only user.
     std::shared_ptr<const KeyCache> mCache;
     std::shared_ptr<NewKeyApprovalDialog> mDialog;
     Qt::WindowFlags mDialogWindowFlags;
+    GpgME::Protocol mPreferredProtocol;
     int mMinimumValidity;
     QString mCompliance;
 };
@@ -616,7 +629,7 @@ void KeyResolver::start(bool showApproval, QWidget *parentWidget)
         if (d->mSign) {
             // So every recipient could be resolved through
             // a combination of PGP and S/MIME do we also
-            // have signining keys for both?
+            // have signing keys for both?
             needsUser |= !(d->mSigKeys.contains(AnyOpenPGP) &&
                            d->mSigKeys.contains(AnySMIME));
         }
@@ -727,6 +740,11 @@ void KeyResolver::setDialogWindowFlags(Qt::WindowFlags flags)
     d->mDialogWindowFlags = flags;
 }
 
+void KeyResolver::setPreferredProtocol(GpgME::Protocol proto)
+{
+    d->mPreferredProtocol = proto;
+}
+
 void KeyResolver::setMinimumValidity(int validity)
 {
     d->mMinimumValidity = validity;
